#!/bin/bash

. bot.properties
input=".bot.cfg"
echo "Starting session: $(date "+[%y:%m:%d %T]")" > $log
echo "NICK $nick" > $input
echo "USER $nick 0 * :$nick" >> $input
echo "MODE $nick +B" >> $input


function save_info {
  now_playing=$(<$npfile)
}

function parse_dj {
  grep -Eo '^\([^)]*\)' $npfile | sed 's/[()]//g' | xargs
}

function msg {
  echo "PRIVMSG ${1} :${2}" >> $input
}

# save info on startup
save_info
dj=$(parse_dj)

# main loop
tail -f $input | telnet $server $port | while read res
do
  # log the session
  echo "$(date "+[%y:%m:%d %T]")$res" >> $log

  # now playing
  if [[ $now_playing != "$(<$npfile)" ]]; then
    save_info
    [[ ! -z "${dj}" ]] && echo -e "PRIVMSG #$channel :\x0303$now_playing" >> $input
  fi

  # new dj!
  if [[ $dj != $(parse_dj) ]]; then
    dj=$(parse_dj)
    if [[ ! -z "${dj}" ]]; then
      for chan in $notify_channels; do
        msg "#$chan" "$dj is now online playing $now_playing! tune in now here: $link"
      done

      while read -u 10 subscriber; do
        msg $subscriber "$dj is now online playing $now_playing! tune in now here: $link"
      done 10< subscribers.txt

      while read -u 11 subscriber email_addr; do
        sed -e "s/<dj>/$dj/g" \
            -e "s|<link>|$link|g" \
            -e "s/<now_playing>/$now_playing/g" \
            -e "s/<email_addr>/$email_addr/g" dj-online-email.tmpl \
          | sendmail $email_addr
      done 11< email-subscribers.txt

      for json in . ./yourtilde ./team; do
        TOOT_JSON_PATH=$json toot "dj $dj is now streaming live on https://tilderadio.org! tune in here: $link"
      done
    fi
  fi

  # do things when you see output
  case "$res" in
    # respond to ping requests from the server
    PING*)
      echo "$res" | sed "s/I/O/" >> $input
    ;;

    # make sure we're joined
    *376*|*404*)
      for chan in $notify_channels; do
        echo "JOIN #$chan" >> $input
      done
    ;;

    # run when a message is seen
    *PRIVMSG*)
      echo "$res"
      who=$(echo "$res" | perl -pe "s/:(.*)\!.*@.*/\1/")
      from=$(echo "$res" | perl -pe "s/.*PRIVMSG (.*[#]?([a-zA-Z]|\-)*) :.*/\1/")
      echo $who $from

      # "#" would mean it's a channel
      if [ "$(echo "$from" | grep '^#')" ]; then
        test "$(echo "$res" | grep ":$nick:")" || continue
        will=$(echo "$res" | perl -pe "s/.*:$nick:(.*)/\1/")
      else
        will=$(echo "$res" | perl -pe "s/.*$nick :(.*)/\1/")
        from=$who
      fi
      will=$(echo "$will" | perl -pe "s/^ //")
      com=$(echo "$will" | cut -d " " -f1)

      case "$com" in
        subscribe)
          if grep -q $who subscribers.txt; then
            msg $from "you're already subscribed! :)"
          else
            echo $who >> subscribers.txt
            msg $from "i'll send you a direct message when a dj starts streaming!"
          fi
        ;;

        unsubscribe)
          sed -i "/$who/d" subscribers.txt
          msg $from "i'll stop sending you updates."
        ;;

        email-subscribe)
          if grep -q $who email-subscribers.txt; then
            msg $from "you're already subscribed! :)"
          else
            email_addr=$(echo "$will" | cut -d " " -f2)
            echo "$who $email_addr" >> email-subscribers.txt
            msg $from "i'll send you an email when a dj starts streaming!"
          fi
        ;;

        email-unsubscribe)
          sed -i "/$who/d" email-subscribers.txt
          msg $from "i'll stop sending you email updates."
        ;;

        time)
          msg $from "$(TZ=UTC date)"
        ;;

        np)
          msg $from "$now_playing"
        ;;

        dj)
          msg $from "${dj:-"no one"} is on the air now"
        ;;

        link)
          msg $from "$link"
        ;;

        source)
          msg $from "$source"
        ;;

        schedule)
          msg $from "$schedule"
        ;;

        help)
          msg $from "hey hi, my commands are subscribe, unsubscribe, np, dj, link, slogan, and schedule. if you prefer email notifications, you can use radiobot: email-subscribe you@example.com"
        ;;

        radio|slogan)
          msg $from "$(shuf -n1 slogans.txt)"
        ;;

        paymybills)
          msg $from "whaddya mean?! i'm broker than you!"
        ;;

        *)
          echo "no command: $res"
        ;;

      esac
    ;;

    # else
    *)
      echo "$res"
    ;;
  esac
done

